import AutoRPE.UtilsRPE.ModifyCodeUtils as ModifyCodeUtils
import AutoRPE.UtilsRPE.Vault as Vault
import AutoRPE.UtilsRPE.VariablePrecision as VariablePrecision


def get_command_line_arguments():
    import argparse
    from AutoRPE.UtilsRPE.Error import PathType
    parser = argparse.ArgumentParser()
    parser.add_argument("--preprocessed-input-files", required=True, type=PathType(exists=True, type="dir"), nargs='+',
                        help="Path to the preprocessed sources")
    parser.add_argument("--original-input-files", required=True, type=PathType(exists=True, type="dir"), nargs='+',
                        help="Path to the non preprocessed sources")
    parser.add_argument("--vault", default=None, type=PathType(exists=True, type="file"),
                        help="For debug purposes")

    return parser.parse_args()


if __name__ == "__main__":
    args = get_command_line_arguments()
    preprocessed_input_files = args.preprocessed_input_files
    original_input_files = args.original_input_files
    vault = args.vault
    # Set working precision to double, since it is the precision used to compile preprocessed sources
    VariablePrecision.set_working_precision('dp')
    if vault is None:
        vault = Vault.create_database(preprocessed_input_files, extension=".f90")
    else:
        # This is useful when debugging: it is not needed to generate a vault at every run
        vault = Vault.load_vault(vault)
    # Set working precision to single, so as to find the needed fixes
    VariablePrecision.set_working_precision('sp')
    # Search in preprocessed code where fixes are due
    dictionary_of_fixes = ModifyCodeUtils.get_fixes(vault)
    # Retrieve these calls in the original code, and fix them
    fixed_modules = ModifyCodeUtils.add_cast(original_input_files, dictionary_of_fixes)
    # Save fixed sources
    for module in fixed_modules:
        module.save_file()
