# Stores pattern reused along the code

# Capures a real number with precision specification: 0.5_dp / 0_sp
real_number_with_precision = r"^[0-9\.]*_[a-zA-Z]+$"
integer_number_with_precision = r"^[0-9\.]*_\d+$"
# Pattern for numbers in exponential form, or numbers with double precision exponent
number_exp_DPE_notation = r'\s*\d+\.?\d*[e|d][+|-]*\d+[\w]*'

#### VARIABLE
variable = r"^\w+$"
#### ARRAY
# Matches simple array, and sliced array
array = r"\w+\s*\(\s*[\w,:\d\s+\-\*=]+\s*\)"
match_array_brackets = r"\w+\s*(\(\s*[\w,:\d\s+\-\*=]+\s*\))"
# array(:)%b
array_with_member = array + r"%\w+"
#  array(:)%b(:)
array_with_array_member = array + "%" + array

###### STRUCTURE
# Simple strucure a%b%c
structure = r"\w+%*\w*%\w+"
# Simple structure with array member a%b(:)
structure_with_member_array = r"\w+%" + array

# Matches with the (/old, style, vec /) or the new [array, style]
# hardcoded_array = r"(\(\/|\[)(\s*[\w\s,%.()\+\-=*]*\s*)(\/\)|\])"
hardcoded_array = r"\s*(\(\/|\[)(.*)(\/\)|\])"
# In write statement there can be a statement like
# WRITE(numout,9402) jj, (ilci(ji,jj), ilcj(ji,jj),ji=il1,il2)
# the second argument has the type of the first el of the array
array_in_write_statement = r"^\s*\([\s\w,\(\):]+=[\s\w,\(\):]+\)"

loop = r',\s*\w+\s*=\s*\d+\s*,'

#### DECLARATION

# Variables declaration
variable_declaration = r'\s*::\s*'

# TYPE

# type(rpe) declaration with assignation
rpe_parameter_declaration = r'^.*type\(rpe_var\).*::.*='

# Given a name match the type declared with this name
type_name_begin = r'\btype\b.*\b%s\b'

# Captures simple types declaration
simple_type_declaration = r"^ *type[^()]*[ \t](\w+) *$"
# Captures just types declaration with extended the attribute
extended_type_declaration = r"^ *type\s*,(\s*public\s*,|)\s*extends *\(\s*(\w*) *\) *:: * (\w*)"
# Captures type with possible public and extended attributes
general_type_declaration = r'^ *type\s*(,|)\s*(public\s*::|public|)\s*(,|)\s*(extends\((\w*)\)\s*::|) * (\w*)'
# Captures end of groups
end_type_statement = r"^\s*END TYPE"
# Captures the variable and value of an assignation
assignation_val = r"^([^\!]*)=([^\!]*)"


# REAL (Can be a variable declaration, or a function ret_val specification)

# All the lines that declare a real
general_real_declaration = r'^\s*(real).*(::|(function))'
# real(wp/sp/dp/[...]) :: var
real_with_precision_declaration = r'[^\!]*(real *\([\w= ]*\)).*(::|(function))'
# real :: var
pure_real_declaration = r'^\s*\w?(real).*(::|(function))'
# REAL((kind=)dp/sp/wp) => group(3) == dp/sp/wp
precision_of_real_declaration = r'real\s*\(\s*((kind\s*=|)\s*(\w+))\s*\)'

# POINTER

# Captures when a pointer is assigned on a line and gives back
# - group 0 => eventual ptr attributes
# - group 2 => ptr
# - group 3 => target
pointers_assignation = r"(.*::|)\s*(.*)=>\s*(.*)"

# SUBPROGRAM

function_declaration = r'function\b(?!(?<=\bend function)\s*)\s+(\w+)'
end_function_declaration = r'^\s*end\s*function'
subprogram_declaration = r'(subroutine|function)\s+\w+\s*\(.*\)'
subroutine_declaration = r'subroutine\b(?!(?<=\bend subroutine)\s*)\s+(\w+)'
subprogram_name_declaration = r"((subroutine|function) +\b)%s\b"
subprogram_name_end_declaration = r"(end\s+(subroutine|function)\s+\b)%s\b"

# CALL
# (?![0-9])[a-z0-9_]+): Match for alphanumeric string with negative lookahead for only numbers
call_to_function = r"(?<!subroutine )(?<!call )[+-]*(\b(?![0-9])[a-z0-9_]+)\s*\("
call_to_subroutine = r"^\s*call\s+(\w+)"
call_to_function_name = r'(?<!subroutine )(?<!call )[+-]*(\b%s\b)\s*\('
call_to_subroutine_name = r'^\s*call\s+%s'
assignation_to_DT = r"(\w+) *\("

# CALL to READ
call_to_read = r'^\s*(\bread\b\s*\(.*)'

# Allocation
allocation_variable_name = r"\ballocate\b\s*\(.*\b%s\b.*"
allocation_member_name = r"\ballocate\b\s*\(.*(?<=%%)\b%s\b.*"
allocation_general = r'\ballocate\b\s*\(.*'

# Reshape:
# - group 0 => The entire call
# - group 1 => the hardcoded array being reshaped
reshape_function = r'RESHAPE\s*\(\s*(\(\/(.*)\/\))\s*,\s*\(\s*\/(.*)\/\)\s*\)'

# Use statement
use_statement = r'^use\s'

# Assignation like a = b
# - group 0 a =
# - group 3 b
assignation = r'(^\s*\w+(\(.*\))*\s*=)\s*(.*)'


def escape_string(string):
    import re
    escaped_string = string
    # In case string does start with a char we'll add the \b to indicate a word border
    if re.match(r"\w", escaped_string.strip()[0]):
        escaped_string = r'\b' + escaped_string.strip()

    # Escape operation signs that means something in  regex patterns
    escaped_string = escaped_string.replace("+", r"\+")
    escaped_string = escaped_string.replace("-", r"\-")
    escaped_string = escaped_string.replace("*", r"\*")
    # For floating point numbers
    escaped_string = escaped_string.replace(".", r"\.")
    if escaped_string.count("("):
        escaped_string = escaped_string.replace("(", r"\s*\(")
        # Avoid matching structures
        if not escaped_string.count("%"):
            escaped_string = escaped_string.replace(")", r"\)(?!%)")
        else:
            escaped_string = escaped_string.replace(")", r"\)")
    if not escaped_string.strip()[-1] == ")":
        escaped_string += r"\b"
    return escaped_string
